package com.ath.flutterservice.flutter_service;

import com.google.protobuf.nano.MessageNano;
import com.yy.gslbsdk.HttpDnsService;
import com.yy.hiidostatis.api.HiidoSDK;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.time.Year;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import io.flutter.Log;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.flutter.view.FlutterView;
import tv.athena.annotation.MessageBinding;
import tv.athena.core.axis.Axis;
import tv.athena.core.sly.Sly;
import tv.athena.service.api.BindResponse;
import tv.athena.service.api.ByteArrayResponse;
import tv.athena.service.api.ConnectStatus;
import tv.athena.service.api.DataResponse;
import tv.athena.service.api.GroupType;
import tv.athena.service.api.IBindCallback;
import tv.athena.service.api.IByteArrayCallback;
import tv.athena.service.api.IDataCallback;
import tv.athena.service.api.IMessageCallback;
import tv.athena.service.api.ISubscribeGroupTypeCallback;
import tv.athena.service.api.ISvcConfig;
import tv.athena.service.api.ISvcService;
import tv.athena.service.api.ITokenProvider;
import tv.athena.service.api.IUnPack;
import tv.athena.service.api.MessageResponse;
import tv.athena.service.api.Service;
import tv.athena.service.api.ServiceFailResult;
import tv.athena.service.api.event.ServiceBroadcastEvent;
import tv.athena.service.api.event.ServiceUnicastEvent;
import tv.athena.service.api.event.SvcStateEvent;
import tv.athena.service.api.hide.IService;
import tv.athena.util.FP;
import tv.athena.util.RuntimeInfo;


/** FlutterServicePlugin */
public class FlutterServicePlugin implements MethodCallHandler , EventChannel.StreamHandler {
    private static final String EVENT_CHANNEL = "flutter_service_event";
    private static final String kServiceStatusBroadcast = "flutter_service_broad_status";
    private static final String kServiceKickOffBroadcast = "flutter_service_broad_kickoff";
    private static final String kServicePushDataBroadcast = "flutter_service_broad_pushdata";
  public  EventChannel.EventSink mEvents;
  /** Plugin registration. */
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "flutter_service");
    final FlutterServicePlugin flutterServicePlugin =  new FlutterServicePlugin();
    channel.setMethodCallHandler(flutterServicePlugin);

    RuntimeInfo.setSAppContext(registrar.context());

    Log.i("service:registerWith",(registrar.messenger()==null)+"");

    final EventChannel eventChannel = new EventChannel(registrar.messenger(), EVENT_CHANNEL);
    eventChannel.setStreamHandler(flutterServicePlugin);


  }

  public FlutterServicePlugin(){
    super();
      Sly.Companion.subscribe(this);


  }
  @Override
  public void onMethodCall(MethodCall call, Result result) {
    Log.i("service:onMethodCall",call.method+",arguments:"+call.arguments);
    if (call.method.equals("getPlatformVersion")){
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    }else if (call.method.equals("initServiceWithParameters")){
      this.initServiceWithParameters(call.arguments,result);
    }else if (call.method.equals("sendRequestWithParameters")){
      this.sendRequestWithParameters((Map) call.arguments,result);
    }else if (call.method.equals("subscribeBroadcastWithParameters")){
      this.subscribeBroadcastWithParameters((Map) call.arguments,result);

    }else if (call.method.equals("unsubscribeBroadcastWithParameters")){
      this.unsubscribeBroadcastWithParameters((Map) call.arguments,result);

    }
    else {
      result.notImplemented();
    }


  }
//  Map params = {
//          "appId": 1249193970, // service appid
//          "compAppid": "igameip", // 公司统一的 appid
//          "version": "2.0", // 应用的版本
//          "region": "AE", //地区/国家码，传空则native层自己取默认值
//          "testIP": "",
//          "testPort": ""
//};
  public void initServiceWithParameters(Object params, Result result ){

    Map map  = (Map)params;
    Integer appid = (Integer) map.get("appId");
    String compAppid = (String) map.get("compAppid");
    String version = (String) map.get("version");
    String region = (String) map.get("region");
    String appidStr =  appid.toString();
    String accountId = (String) map.get("accountId");
    if (accountId!=null){
        HttpDnsService.getService(RuntimeInfo.sAppContext, accountId, null,
                HiidoSDK.instance().getDeviceId(RuntimeInfo.sAppContext), "CN");
    }else {
        HttpDnsService.getService();
    }

    ISvcConfig config = Axis.Companion.getService(ISvcService.class).config();
    config.setAppId(Long.parseLong(appidStr)).setAreaCode(region).apply();

//    Sly.Companion.subscribe(this);

    Service.setKickByService(new IBindCallback() {
      @Override
      public void onBindSuccess(@NotNull BindResponse bindResponse) {
          if ( FlutterServicePlugin.this.mEvents!=null){

          }
      }

      @Override
      public void onBindFail(@NotNull ServiceFailResult serviceFailResult, @Nullable Exception e) {
          if ( FlutterServicePlugin.this.mEvents!=null){

          }
      }

      @Override
      public void onKickOut(long l, int i, @NotNull String s) {
        if (FlutterServicePlugin.this.mEvents!=null){
          Map broadcastData = new HashMap();
          broadcastData.put("uid",l);
          broadcastData.put("code",i);
          broadcastData.put("desc",s);

          //
          Map map = new HashMap();
          map.put("broadcastName",kServiceKickOffBroadcast);
          map.put("broadcastData",broadcastData);
          FlutterServicePlugin.this.mEvents.success(map);
        }
      }
    });
    result.success(true);
  }
  public void sendRequestWithParameters(Map params, final Result result){
//      Sly.Companion.postMessage(new SvcStateEvent(ConnectStatus.INIT));
    String mServerName = (String) params.get("mServerName");
    String mFunctionName = (String) params.get("mFunctionName");
    String mProtoType = (String) params.get("mProtoType");
    String mTranceid = (String) params.get("mTranceid");
    Map mClientHeaders = (Map) (params.get("mClientHeaders")!=null?params.get("mClientHeaders"):new HashMap<>());
    byte[] mRequestData = (byte[]) params.get("mRequestData");
    String mBusinessContext = (String) params.get("mBusinessContext");
    mProtoType = !FP.empty(mProtoType)?mProtoType:"protobuf";
    mTranceid = !FP.empty(mTranceid)?mProtoType:"";
    Service.send(mBusinessContext, mServerName, mFunctionName, mRequestData,mProtoType,mTranceid, mClientHeaders, new IDataCallback() {
      @Override
      public void onMessageSuccess(@NotNull DataResponse dataResponse) {
          Log.i("sendRequestWithParameters response","success"+dataResponse.getDescption());
         Map<String,Object> map = new HashMap<String,Object>();
         map.put("mServerName",dataResponse.getServerName());
         map.put("mFunctionName",dataResponse.getFuncName());
         map.put("mResponseData",dataResponse.getMessage());
         map.put("mResCode",dataResponse.getResultCode());
         map.put("mServerHeaders",dataResponse.getServerHeader());
         map.put("mBusinessContext",dataResponse.getContext());
         map.put("mResMsg",dataResponse.getDescption());
         map.put("mTraceId",dataResponse.getTraceId());
         map.put("mProtoType",dataResponse.getPrototype());
         result.success(map);
      }

      @Override
      public void onMessageFail(@NotNull ServiceFailResult serviceFailResult, @Nullable Exception e) {
        Log.i("sendRequestWithParameters response","fail"+serviceFailResult.getDescription());
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("error",serviceFailResult.getDescription());
        map.put("sdkResCode",serviceFailResult.getSdkCode());
        map.put("srvResCode",serviceFailResult.getSvrCode());
        map.put("reqId",serviceFailResult.getRequestId());
        result.success(map);

      }
    });
  }
  private String numToStr(Object num){
      String str = null;
      if (num.getClass().equals(Integer.class)){
          Integer mGrpTypeInt = (Integer) num;
          str =  mGrpTypeInt.toString();
      }
      if (num.getClass().equals(Long.class)){
          Long mGrpTypeLong = (Long) num;
          str =  mGrpTypeLong.toString();
      }
      return  str;
  }
public void subscribeBroadcastWithParameters(Map params, final Result result){
    Long a = new Long("1");

  ArrayList<Map> rawUserGroups =(ArrayList) params.get("mUserGrps");

  if (rawUserGroups.size() == 0) {
     return;
  }
   ArrayList<GroupType> groupTypes = new ArrayList<>();
  for (int i =0;i<rawUserGroups.size();i++){
      Map groups = rawUserGroups.get(i);
      Object mGrpTypeObj = groups.get("mGrpType");
      Object mGrpIdObj = groups.get("mGrpId");
      String mGrpTypeStr = null;
      String mGrpIdStr = null;
      if (mGrpTypeObj.getClass().equals(Integer.class)){
          Integer mGrpTypeInt = (Integer) mGrpTypeObj;
          mGrpTypeStr =  mGrpTypeInt.toString();
       }
      if (mGrpTypeObj.getClass().equals(Long.class)){
          Long mGrpTypeLong = (Long) mGrpTypeObj;
           mGrpTypeStr =  mGrpTypeLong.toString();
      }
      if (mGrpIdObj.getClass().equals(Integer.class)){
          Integer mGrpIdObjInt = (Integer) mGrpIdObj;
          mGrpIdStr =  mGrpIdObjInt.toString();
      }
      if (mGrpIdObj.getClass().equals(Long.class)){
          Long mGrpIdObjLong = (Long) mGrpIdObj;
          mGrpIdStr =  mGrpIdObjLong.toString();
      }
      Log.i("sub",mGrpTypeObj.getClass()+","+(mGrpTypeObj.getClass().equals(Integer.class))+","+mGrpTypeObj.getClass().equals(Long.class));
      Log.i("sub",mGrpIdObj.getClass()+","+mGrpIdObj.getClass().equals(Integer.class)+","+mGrpIdObj.getClass().equals(Long.class));

      Long.parseLong(mGrpTypeStr);
      Log.i("sub",mGrpTypeStr+","+mGrpIdStr+"。");
      Log.i("sub",mGrpTypeStr.getClass()+","+mGrpIdStr.getClass());
     GroupType groupType = new GroupType(Long.parseLong(mGrpTypeStr),Long.parseLong(mGrpIdStr));
     groupTypes.add(groupType);
  }


    Service.subscribeBroadcast(groupTypes, new ISubscribeGroupTypeCallback() {
      @Override
      public void onSuccess(@NotNull String s, int i) {
          Log.i("sub","onSuccess:"+i);

          Map<String,Object> map = new HashMap<String,Object>();

        map.put("reqId",0);//这里暂时athservive没有，暂时为空
        map.put("mBusinessContext",s);
        map.put("mResCode",new Integer(i));
        result.success(map);
      }

      @Override
      public void onFail(@NotNull ServiceFailResult serviceFailResult, @Nullable Exception e) {
          Log.i("sub","onFail:"+serviceFailResult.getDescription());

          Map<String,Object> map = new HashMap<String,Object>();
          map.put("reqId",serviceFailResult.getRequestId());//这里暂时athservive没有，暂时为空
          map.put("error",serviceFailResult.getDescription());
          map.put("sdkResCode",Long.valueOf(serviceFailResult.getSdkCode()));
          map.put("srvResCode",Long.valueOf(serviceFailResult.getSvrCode()));
          result.success(map);

      }
    });

}
 public void unsubscribeBroadcastWithParameters(Map params, final Result result){
   ArrayList<Map> rawUserGroups =(ArrayList) params.get("mUserGrps");

   if (rawUserGroups.size() == 0) {
     return;
   }
   ArrayList<GroupType> groupTypes = new ArrayList<>();
   for (int i =0;i<rawUserGroups.size();i++){
       Map groups = rawUserGroups.get(i);
       Object mGrpTypeObj = groups.get("mGrpType");
       Object mGrpIdObj = groups.get("mGrpId");
       String mGrpTypeStr = this.numToStr(mGrpTypeObj);
       String mGrpIdStr = this.numToStr(mGrpIdObj);
     GroupType groupType = new GroupType(Long.parseLong(mGrpTypeStr),Long.parseLong(mGrpIdStr));
     groupTypes.add(groupType);
   }
   Service.unsubscribeBroadcast(groupTypes, new ISubscribeGroupTypeCallback() {
     @Override
     public void onSuccess(@NotNull String s, int i) {
       Map<String,Object> map = new HashMap<String,Object>();
         Log.i("unsub","onSuccess:"+i);
       map.put("reqId",0);//这里暂时athservive没有，暂时为空
       map.put("mBusinessContext",s);
       map.put("mResCode",i);
       result.success(map);
     }

     @Override
     public void onFail(@NotNull ServiceFailResult serviceFailResult, @Nullable Exception e) {
       Log.i("unsub","onFail:"+serviceFailResult.getDescription());
       Map<String,Object> map = new HashMap<String,Object>();
       map.put("reqId",serviceFailResult.getRequestId());//这里暂时athservive没有，暂时为空
       map.put("error",serviceFailResult.getDescription());
       map.put("sdkResCode",serviceFailResult.getSdkCode());
       map.put("srvResCode",serviceFailResult.getSvrCode());
       result.success(map);
     }
   });
 }
 public void  bind(Map params, final Result result){
    Service.bind(0, 1, new ITokenProvider() {
     @NotNull
     @Override
     public String getToken(long l) {
       return null;
     }
   }, new IBindCallback() {
     @Override
     public void onBindSuccess(@NotNull BindResponse bindResponse) {

     }

     @Override
     public void onBindFail(@NotNull ServiceFailResult serviceFailResult, @Nullable Exception e) {

     }

     @Override
     public void onKickOut(long l, int i, @NotNull String s) {

     }
   });
 }
//  fun bind(uid: Long, tokenType: Int, tokenProvider: ITokenProvider, callback: IBindCallback)
//
//  fun unbind(callback: IUnbindCallback)

  @MessageBinding
  public void onServiceBroadcastEvent(ServiceBroadcastEvent event) {
      Log.i("onServiceBroadcastEvent","event:"+event.getFuncName());

      if (this.mEvents!=null){
      Map broadcastData = new HashMap();
      broadcastData.put("uid",event.getUid());
      broadcastData.put("groupType",event.getGroupType());
      broadcastData.put("groupId",event.getGroupId());
      broadcastData.put("server",event.getServerName());
      broadcastData.put("function",event.getFuncName());
      broadcastData.put("prototype","");//这里暂时athservive没有，暂时为空
      broadcastData.put("data",event.getMessage());

      //
      Map map = new HashMap();
      map.put("broadcastName",kServicePushDataBroadcast);
      map.put("broadcastData",broadcastData);
      this.mEvents.success(map);
    }
  }
  @MessageBinding
  public void onServiceUnicastEvent(ServiceUnicastEvent event) {
      Log.i("onServiceUnicastEvent","event:"+event.getFuncName());
      if (this.mEvents!=null){
      Map broadcastData = new HashMap();
      broadcastData.put("uid",event.getUid());
//      broadcastData.put("groupType",event.getGroupType());//这里暂时athservive没有，暂时为空
//      broadcastData.put("groupId",event.getGroupId());
      broadcastData.put("server",event.getServerName());
      broadcastData.put("function",event.getFuncName());
      broadcastData.put("prototype","");//这里暂时athservive没有，暂时为空
      broadcastData.put("data",event.getMessage());

      //
      Map map = new HashMap();
      map.put("broadcastName",kServicePushDataBroadcast);
      map.put("broadcastData",broadcastData);
      this.mEvents.success(map);
  }
  }
  @MessageBinding
  public void onSvcStateEvent(SvcStateEvent event) {
      Log.i("onSvcStateEvent","event:"+event.getStatus());
      if (this.mEvents!=null){
       Map map = new HashMap();
       map.put("broadcastName",kServiceStatusBroadcast);
       map.put("broadcastData",Integer.valueOf(event.getStatus().getStatus()));
       this.mEvents.success(map);
     }
  }

  @Override
  public void onListen(Object o, EventChannel.EventSink eventSink) {
    this.mEvents = eventSink;

  }

  @Override
  public void onCancel(Object o) {
    this.mEvents = null;

  }
}
